###########################################################################
#
#   Copyright (c) 2000, 2001 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000, 2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.10 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import OFS.Folder
import OFS.DTMLMethod
import address_book_contact
import product_base_item
import base_item
import string
import address_book_group_container
import address_book_contact_container

manage_add_address_book_form = HTMLFile('address_book_add', globals())


def manage_add_address_book(self, id, title, install_default_html='',
				install_default_theme='',
				install_default_id_generator='',
				REQUEST=None, RESPONSE=None):
	"""Add an instance of the address_book class."""

	self._setObject(id, address_book(id, title,
				install_default_html=install_default_html,
				install_default_theme=install_default_theme,
				install_default_id_generator=install_default_id_generator
			))

	if RESPONSE is not None:

		return RESPONSE.redirect(self.absolute_url() + '/' + id)

	else:

		try:

			return self[id]

		# If we're adding instances on the root.

		except KeyError:

			return None


class dummy_group:

	def __init__(self):

		self.id = ''
		self.name = 'No specific group'


def name_match(object, letter):
	"""Filters out objects somewhat intelligently."""

	name = object.surname

	if not string.strip(name):

		name = object.firstname

	if string.strip(name):

		return string.upper(name[0]) == letter

	else:

		return None


class address_book(
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	product_base_item.product_base_item,
	base_item.base_item
	):
	"""For managing contacts."""

	product_name = 'AddressBook'

	meta_type = 'AddressBook'
	icon = 'misc_/AddressBook/Folder_icon.gif'

	use_local_navigation = 1
	use_local_preferences = 1

	letters = ['A','B','C','D','E','F','G','H','I','J','K','L','M',
			'N','O','P','Q','R','S','T','U','V','W','X','Y','Z']

	_properties = (
		{'id':'title', 'type':'string'},
		{'id':'default_group', 'type':'string'}
		)

	meta_types = (
		# HACK, FIXME
		{'name':'AddressBookContact',
			'action':'contacts/manage_add_address_book_contact_form'},
		)

	manage_add_address_book_contact_container = \
			address_book_contact_container.manage_add_address_book_contact_container

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('Add AddressBookContact',
		 ('manage_add_address_book_contact', 'manage_add_address_book_contact_form')),
		('Use AddressBook',
		 ('', 'address_book_edit', 'list_names', 'display',
			'sort_names', 'get_address_book_contacts',
			'get_contacts_from_group', 'get_contact_listing', 'get_all_groups',
			'render_group_widget', 'get_address_book_letters',
			)),
		)

	index_html = HTMLFile('address_book_index', globals())
	address_book_edit = base_edit = HTMLFile('address_book_edit', globals())

	list_names = HTMLFile('address_book_list_names', globals())
	display = HTMLFile('address_book_display', globals())


	def sort_names(self, name_starts_with):
		"""Returns a sorted list."""

		matched_contacts = []

		for contact in self.get_address_book_contacts():

			if name_match(contact, name_starts_with):

				matched_contacts.append(contact)

		return matched_contacts


	def get_address_book_contact_containers(self):
		"""Returns a list of the contact containers contained."""

		return self.objectValues('AddressBookContactContainer')


	def get_address_book_contacts(self):
		"""Returns a list of all contact instances in the address book."""

		contacts = []

		for address_book_contact_container in self.get_address_book_contact_containers():

			map(contacts.append, address_book_contact_container.get_contacts())

		return contacts


	def get_contacts_from_group(self, group):
		"""Returns contacts for given group."""

		# We make sure this isn't python executable code.

		group = int(group)

		filter_function = eval("""lambda contact: '%s' in contact.groups""" % group)

		contacts = filter(filter_function, self.get_address_book_contacts())

		return contacts


	def get_selected_contacts(self, ids):
		"""Returns a list of the contacts selected."""

		contacts = []

		if type(ids) is not type([]):

			ids = [ids]

		for id in ids:

			contacts.append(self[id])

		return contacts


	def get_contact_listing(self, REQUEST):
		"""Returns contacts from groups selected."""

		if REQUEST.has_key('n') and REQUEST['n']:

			return self.sort_names(REQUEST['n'])

		contacts = []

		if REQUEST.has_key('group') and REQUEST['group']:

			groups = REQUEST['group']

		else:

			return self.get_address_book_contacts()

		if not type(groups) == type([]):

			groups = [groups]

		for group in groups:

			contacts = contacts + self.get_contacts_from_group(group)

		return contacts


	def get_all_groups(self):
		"""Returns all known groups."""

		return self.get_contact_groups()


	def get_contact_groups(self):
		"""Returns all known groups."""

		return self.contact_groups.objectValues('AddressBookGroup')


	def render_group_widget(self, REQUEST=None, group=None, name='group', size=4):
		"""Returns an HTML widget for selecting groups."""

		if REQUEST is not None and REQUEST.has_key('group'):

			selected_group = REQUEST['group']

		elif group is not None:

			selected_group = group

		else:

			selected_group = ''			

		select_widget = """<select name="%s" size="%s" multiple>""" % (name, size)

		for group in [dummy_group()] + self.get_all_groups():

			if selected_group == group.id:

				selected = " selected"

			else:

				selected = ""

			select_widget = select_widget + """<option value="%s"%s>%s</option>""" % \
						(group.id, selected, group.name)

		return select_widget + "</select>"


	def get_address_book_letters(self):
		"""Returns a list of address book letters."""

		return self.letters		


	def __init__(self, id, title='',
			install_default_html=None,
			install_default_theme=None,
			install_default_id_generator=None):
		"""Initialize the class instance."""

		self.id = id
		self.title = title
		self.default_group = '-1'

		self.counter = 0

		address_book_group_container.manage_add_address_book_group_container(self)
		address_book_contact_container.manage_add_address_book_contact_container(self)

		print self, type(self)

		product_base_item.utils.default_installation(
			self,
			install_default_html=install_default_html,
			install_default_theme=install_default_theme,
			install_default_id_generator=install_default_id_generator
				)


default__class_init__(address_book)
